/**
 * synopsis: Generates an Fvwm menu segment based on an XDG menu structure
 * purpose: The purpose of this utility is to keep the Fvwm menu
 *          synchronized with the XDG configuration
 * usage: fvwm-xdgmenu <xdgMenuFile>
 * author: Raul Suarez (rarsa at yahoo dot com)
 * Licence: GPL
 *
 * Dependencies:
 *   gnome-menus 2.12
 *   glib
 */

#include <glib.h>
#include <glib/gprintf.h>
#include <gmenu-tree.h>


/*=============================================================================
 * Declarations
 */
static void show_help();
static void process_directory(GMenuTreeDirectory *directory, char *prefix);
static void add_popup(GMenuTreeDirectory *directory, char *prefix);
static void add_entry(GMenuTreeEntry *entry);
static void add_separator(GMenuTreeSeparator *entry);


/*=============================================================================
 * Main Function
 */
int
main (int argc, char **argv)
{

  if ((argc != 2)
      || (strcmp (argv[1], "-h") == 0)
      || (strcmp (argv[1], "-help") == 0)
      || (strcmp (argv[1], "--help") == 0))
  {
    show_help();
    return (1);
  }

  GMenuTree *menuTree = gmenu_tree_lookup (argv[1],  GMENU_TREE_FLAGS_NONE);

  GMenuTreeDirectory *rootDirectory = gmenu_tree_get_root_directory(menuTree);

  process_directory(rootDirectory, "");

  gmenu_tree_item_unref (rootDirectory);

  return 0;

}

/*=============================================================================
 * Shows usage parameters
 */
void
show_help()
{
  printf ("Creates a Fvwm menu from an XDG menu structure.\n");
  printf ("Usage:\n");
  printf ("  Fvwm-xdgmenu xdgmenufile\n");
  printf ("\n");
  printf ("  xdgmenufile: Fully qualified path to the XDG menu file. This is a mandatory parameter\n");
  printf ("\n");
  printf ("For example:\n");
  printf ("  fvwm-xdgmenu \"/etc/xdg/menus/applications.menu\"\n\n");
  printf ("\n");
  printf ("to invoke it from Fvwm, add a menuprog entry to the menu configuration file\n\n");
  printf ("  menuprog Title icon fvwm-xdgmenu xdgmenufile\n");
}

/*=============================================================================
 * This function processes a directory entry and all it's child nodes
 */
void
process_directory(GMenuTreeDirectory *directory, char *prefix)
{
  int hasSeparator = 0;

  char *directoryName = g_strconcat(prefix,gmenu_tree_directory_get_name(directory),NULL);
  g_strdelimit (directoryName, " /", '-');
  g_printf("\n\nDestroyMenu \"%s\" \n", directoryName);
  g_printf("AddToMenu \"%s\" \n", directoryName);

  GMenuTreeItemType entryType;

  GSList *entryList = gmenu_tree_directory_get_contents (directory);

  GSList *l;

  for (l = entryList; l; l = l->next)
  {

    GMenuTreeItem *item = l->data;

    entryType = gmenu_tree_item_get_type (GMENU_TREE_ITEM(item));

	switch (entryType)
	{
		case GMENU_TREE_ITEM_DIRECTORY:
		  if (hasSeparator)
		  {
				add_separator(GMENU_TREE_SEPARATOR(item));
				hasSeparator = 0;
		  }
      add_popup(GMENU_TREE_DIRECTORY(item), directoryName);
			break;
		case GMENU_TREE_ITEM_ENTRY:
		  if (hasSeparator)
		  {
				add_separator(GMENU_TREE_SEPARATOR(item));
				hasSeparator = 0;
		  }
      add_entry(GMENU_TREE_ENTRY(item));
			break;
		case GMENU_TREE_ITEM_SEPARATOR:
			hasSeparator = 1;
			break;
	}

  }

  for (l = entryList; l; l = l->next)
  {

    GMenuTreeItem *item = l->data;

    entryType = gmenu_tree_item_get_type (GMENU_TREE_ITEM(item));

    if (entryType == GMENU_TREE_ITEM_DIRECTORY)
    {
      process_directory(GMENU_TREE_DIRECTORY(item), directoryName);
    }

    gmenu_tree_item_unref (item);

  }
  free(directoryName);

  g_slist_free (entryList);
}

/*=============================================================================
 * This function adds a popup entry to the menu
 */
void
add_popup(GMenuTreeDirectory *directory, char *prefix)
{
  char *directoryName = g_strconcat(prefix,gmenu_tree_directory_get_name(directory),NULL);
  g_strdelimit (directoryName, " /", '-');
  g_printf("+ \"%s%%%s%%\"  Popup \"%s\" \n",
            gmenu_tree_directory_get_name(directory),
            gmenu_tree_directory_get_icon(directory),
            directoryName);
  free(directoryName);
}

/*=============================================================================
 * This function adds an application entry
 */
void
add_entry(GMenuTreeEntry *entry)
{
  g_printf("+ \"%s%%%s%%\"  Exec %s \n",
            gmenu_tree_entry_get_name(entry),
            gmenu_tree_entry_get_icon (entry),
            gmenu_tree_entry_get_exec (entry));
}

/*=============================================================================
 * This function adds a separator
 */
void
add_separator(GMenuTreeSeparator *entry)
{

  g_printf("+ \"\"					Nop\n");
}

/*=============================================================================
 */
